@@ -15,12 +15,10 @@ from tensorflow.python.ops.custom_gradient import _graph_mode_decorator
 is_tf_keras = strtobool(os.environ.get('TF_KERAS', '0'))
 
 if is_tf_keras:
-    import tensorflow.keras as keras
-    import tensorflow.keras.backend as K
-    sys.modules['keras'] = keras
-else:
-    import keras
-    import keras.backend as K
+    sys.modules['keras'] = tf.keras
+
+import keras
+import keras.backend as K
 
 # 判断是否启用重计算（通过时间换空间）
 do_recompute = strtobool(os.environ.get('RECOMPUTE', '0'))
@@ -135,6 +133,20 @@ def search_layer(inputs, name, exclude_from=None):
                     return layer
 
 
+def align(tensor, axes, ndim=None):
+    """重新对齐tensor（批量版expand_dims）
+    axes：原来的第i维对齐新tensor的第axes[i]维；
+    ndim：新tensor的维度。
+    """
+    assert len(axes) == K.ndim(tensor)
+    assert ndim or min(axes) >= 0
+    ndim = ndim or max(axes) + 1
+    indices = [None] * ndim
+    for i in axes:
+        indices[i] = slice(None)
+    return tensor[indices]
+
+
 def sequence_masking(x, mask, value=0, axis=None):
     """为序列条件mask的函数
     mask: 形如(batch_size, seq_len)的0-1矩阵；
@@ -158,11 +170,8 @@ def sequence_masking(x, mask, value=0, axis=None):
         elif axis < 0:
             axis = K.ndim(x) + axis
         assert axis > 0, 'axis must be greater than 0'
-        for _ in range(axis - 1):
-            mask = K.expand_dims(mask, 1)
+        mask = align(mask, [0, axis], K.ndim(x))
         value = K.cast(value, K.dtype(x))
-        for _ in range(K.ndim(x) - K.ndim(mask)):
-            mask = K.expand_dims(mask, K.ndim(mask))
         x = x * mask + value * (1 - mask)
         if x_dtype == 'bool':
             x = K.cast(x, 'bool')
@@ -232,6 +241,24 @@ def leaky_relu(x, alpha=0.2):
     return tf.nn.leaky_relu(x, alpha=alpha)
 
 
+def attention_normalize(a, axis=-1, method='softmax'):
+    """不同的注意力归一化方案
+    softmax：常规/标准的指数归一化；
+    squared_relu：来自 https://arxiv.org/abs/2202.10447 ；
+    softmax_plus：来自 https://kexue.fm/archives/8823 。
+    """
+    if method == 'softmax':
+        return K.softmax(a, axis=axis)
+    else:
+        mask = K.cast(a > -K.infinity() / 10, K.floatx())
+        l = K.maximum(K.sum(mask, axis=axis, keepdims=True), 1)
+        if method == 'squared_relu':
+            return K.relu(a)**2 / l
+        elif method == 'softmax_plus':
+            return K.softmax(a * K.log(l) / np.log(512), axis=axis)
+    return a
+
+
 class Sinusoidal(keras.initializers.Initializer):
     """Sin-Cos位置向量初始化器
     来自：https://arxiv.org/abs/1706.03762
@@ -249,6 +276,29 @@ class Sinusoidal(keras.initializers.Initializer):
         return embeddings
 
 
+def apply_rotary_position_embeddings(sinusoidal, *tensors):
+    """应用RoPE到tensors中
+    其中，sinusoidal.shape=[b, n, d]，tensors为tensor的列表，而
+    tensor.shape=[b, n, ..., d]。
+    """
+    assert len(tensors) > 0, 'at least one input tensor'
+    assert all([
+        K.int_shape(tensor) == K.int_shape(tensors[0]) for tensor in tensors[1:]
+    ]), 'all tensors must have the same shape'
+    ndim = K.ndim(tensors[0])
+    sinusoidal = align(sinusoidal, [0, 1, -1], ndim)
+    cos_pos = K.repeat_elements(sinusoidal[..., 1::2], 2, -1)
+    sin_pos = K.repeat_elements(sinusoidal[..., ::2], 2, -1)
+    outputs = []
+    for tensor in tensors:
+        tensor2 = K.stack([-tensor[..., 1::2], tensor[..., ::2]], ndim)
+        tensor2 = K.reshape(tensor2, K.shape(tensor))
+        outputs.append(tensor * cos_pos + tensor2 * sin_pos)
+    if len(outputs) == 1:
+        outputs = outputs[0]
+    return outputs
+
+
 def multilabel_categorical_crossentropy(y_true, y_pred):
     """多标签分类的交叉熵
     说明：
@@ -261,16 +311,45 @@ def multilabel_categorical_crossentropy(y_true, y_pred):
         4. 详情请看：https://kexue.fm/archives/7359 。
     """
     y_pred = (1 - 2 * y_true) * y_pred
-    y_pred_neg = y_pred - y_true * K.infinity()
-    y_pred_pos = y_pred - (1 - y_true) * K.infinity()
+    y_neg = y_pred - y_true * K.infinity()
+    y_pos = y_pred - (1 - y_true) * K.infinity()
     zeros = K.zeros_like(y_pred[..., :1])
-    y_pred_neg = K.concatenate([y_pred_neg, zeros], axis=-1)
-    y_pred_pos = K.concatenate([y_pred_pos, zeros], axis=-1)
-    neg_loss = tf.reduce_logsumexp(y_pred_neg, axis=-1)
-    pos_loss = tf.reduce_logsumexp(y_pred_pos, axis=-1)
+    y_neg = K.concatenate([y_neg, zeros], axis=-1)
+    y_pos = K.concatenate([y_pos, zeros], axis=-1)
+    neg_loss = K.logsumexp(y_neg, axis=-1)
+    pos_loss = K.logsumexp(y_pos, axis=-1)
     return neg_loss + pos_loss
 
 
+def sparse_multilabel_categorical_crossentropy(y_true, y_pred, mask_zero=False):
+    """稀疏版多标签分类的交叉熵
+    说明：
+        1. y_true.shape=[..., num_positive]，
+           y_pred.shape=[..., num_classes]；
+        2. 请保证y_pred的值域是全体实数，换言之一般情况下
+           y_pred不用加激活函数，尤其是不能加sigmoid或者
+           softmax；
+        3. 预测阶段则输出y_pred大于0的类；
+        4. 详情请看：https://kexue.fm/archives/7359 。
+    """
+    zeros = K.zeros_like(y_pred[..., :1])
+    y_pred = K.concatenate([y_pred, zeros], axis=-1)
+    if mask_zero:
+        infs = zeros + K.infinity()
+        y_pred = K.concatenate([infs, y_pred[..., 1:]], axis=-1)
+    y_pos_2 = batch_gather(y_pred, y_true)
+    y_pos_1 = K.concatenate([y_pos_2, zeros], axis=-1)
+    if mask_zero:
+        y_pred = K.concatenate([-infs, y_pred[..., 1:]], axis=-1)
+        y_pos_2 = batch_gather(y_pred, y_true)
+    pos_loss = K.logsumexp(-y_pos_1, axis=-1)
+    all_loss = K.logsumexp(y_pred, axis=-1)
+    aux_loss = K.logsumexp(y_pos_2, axis=-1) - all_loss
+    aux_loss = K.clip(1 - K.exp(aux_loss), K.epsilon(), 1)
+    neg_loss = all_loss + K.log(aux_loss)
+    return pos_loss + neg_loss
+
+
 def symbolic(f):
     """恒等装饰器（兼容旧版本keras用）
     """
@@ -357,10 +436,13 @@ def recompute_grad(call):
 # 给旧版keras新增symbolic（装饰器），以兼容optimizers.py
 K.symbolic = getattr(K, 'symbolic', None) or symbolic
 
+# 给tf.keras补充上logsumexp
+K.logsumexp = getattr(K, 'logsumexp', None) or tf.math.reduce_logsumexp
+
 # 添加到 keras.backend 上，使其可以像 K.epsilon() 那样操作
 K.infinity = infinity
 K.set_infinity = set_infinity
-sys.modules['keras.backend'] = K
+sys.modules['tensorflow.keras.backend'] = K
 
 custom_objects = {
     'gelu_erf': gelu_erf,
@@ -371,6 +453,7 @@ custom_objects = {
     'leaky_relu': leaky_relu,
     'Sinusoidal': Sinusoidal,
     'multilabel_categorical_crossentropy': multilabel_categorical_crossentropy,
+    'initializer': keras.initializers.glorot_uniform,  # 就当是默认初始化方案吧
 }
 
 keras.utils.get_custom_objects().update(custom_objects)
