@@ -6,7 +6,7 @@ from bert4keras.backend import get_available_gpus
 from bert4keras.layers import *
 from bert4keras.snippets import insert_arguments
 from bert4keras.snippets import delete_arguments
-from bert4keras.snippets import is_string
+from bert4keras.snippets import is_string, string_matching
 from bert4keras.snippets import orthogonally_resize
 from keras.models import Model
 import json
@@ -217,7 +217,7 @@ class Transformer(object):
             self.input = inputs[0]
 
     def set_outputs(self, outputs):
-        """设置output和oututs属性
+        """设置output和outputs属性
         """
         if not isinstance(outputs, list):
             outputs = [outputs]
@@ -678,7 +678,9 @@ class BERT(Transformer):
                 arguments={'mode': 'dense'},
                 name='Embedding-Token'
             )
-            x = self.apply(inputs=x, layer=BiasAdd, name='MLM-Bias')
+            x = self.apply(
+                inputs=x, layer=ScaleOffset, scale=False, name='MLM-Bias'
+            )
             mlm_activation = 'softmax' if self.with_mlm is True else self.with_mlm
             x = self.apply(
                 inputs=x,
@@ -1196,18 +1198,230 @@ class RoFormer(NEZHA):
         """
         if self.position_bias is None:
 
-            x = inputs
+            if self.custom_position_ids:
+                x = [inputs, self.inputs[2]]
+            else:
+                x = inputs
+
             self.position_bias = self.apply(
                 inputs=x,
                 layer=SinusoidalPositionEmbedding,
                 output_dim=self.attention_key_size,
                 merge_mode='zero',
+                custom_position_ids=self.custom_position_ids,
                 name='Embedding-Rotary-Position'
             )
 
         return self.position_bias
 
 
+class RoFormerV2(RoFormer):
+    """RoFormerV2
+    改动：去掉bias，简化Norm，优化初始化等。
+    """
+    def initializer(self, shape, dtype=None, order=2, gain=1.0):
+        """使用截断正态分布初始化
+        """
+        if shape[0] > 10000 or shape[0] < 10:
+            hidden_size = shape[1]
+        else:
+            hidden_size = shape[0]
+        gain *= self.num_hidden_layers**(-1. / order)
+        stddev = 1.13684723 / hidden_size**0.5 * gain
+        return K.truncated_normal(shape, stddev=stddev)
+
+    def apply_embeddings(self, inputs):
+        """RoFormerV2的embedding是token、segment两者embedding之和
+        """
+        inputs = inputs[:]
+        x = inputs.pop(0)
+        if self.segment_vocab_size > 0:
+            s = inputs.pop(0)
+
+        x = self.apply(
+            inputs=x,
+            layer=Embedding,
+            input_dim=self.vocab_size,
+            output_dim=self.embedding_size,
+            embeddings_initializer=self.initializer,
+            mask_zero=True,
+            name='Embedding-Token'
+        )
+        if self.segment_vocab_size > 0:
+            if self.shared_segment_embeddings:
+                name = 'Embedding-Token'
+            else:
+                name = 'Embedding-Segment'
+            s = self.apply(
+                inputs=s,
+                layer=Embedding,
+                input_dim=self.segment_vocab_size,
+                output_dim=self.embedding_size,
+                embeddings_initializer=self.initializer,
+                name=name
+            )
+            x = self.apply(
+                inputs=[x, s], layer=Add, name='Embedding-Token-Segment'
+            )
+        x = self.apply(
+            inputs=x,
+            layer=Dropout,
+            rate=self.dropout_rate,
+            name='Embedding-Dropout'
+        )
+        x = self.apply(
+            inputs=x,
+            layer=LayerNormalization,
+            zero_mean=False,
+            scale=False,
+            offset=False,
+            name='Embedding-Norm'
+        )
+        if self.embedding_size != self.hidden_size:
+            x = self.apply(
+                inputs=x,
+                layer=Dense,
+                units=self.hidden_size,
+                use_bias=False,
+                kernel_initializer=self.initializer,
+                name='Embedding-Mapping'
+            )
+
+        return x
+
+    def apply_main_layers(self, inputs, index):
+        """RoFormerV2的主体是基于Self-Attention的模块
+        顺序：Att  --> Add --> LN --> FFN --> Add --> LN
+        """
+        x = inputs
+
+        attention_name = 'Transformer-%d-MultiHeadSelfAttention' % index
+        feed_forward_name = 'Transformer-%d-FeedForward' % index
+        attention_mask = self.compute_attention_bias(index)
+        position_bias = self.compute_position_bias(x)
+
+        # Self Attention
+        xi = x
+        x = [x, x, x, position_bias]
+        arguments = {'a_bias': None, 'p_bias': 'rotary'}
+        if attention_mask is not None:
+            arguments['a_bias'] = True
+            x.insert(3, attention_mask)
+        x = self.apply(
+            inputs=x,
+            layer=MultiHeadAttention,
+            arguments=arguments,
+            heads=self.num_attention_heads,
+            head_size=self.attention_head_size,
+            out_dim=self.hidden_size,
+            key_size=self.attention_key_size,
+            use_bias=False,
+            attention_dropout=self.attention_dropout_rate,
+            kernel_initializer=self.initializer,
+            name=attention_name
+        )
+        x = self.apply(
+            inputs=x,
+            layer=Dropout,
+            rate=self.dropout_rate,
+            name='%s-Dropout' % attention_name
+        )
+        x = self.apply(
+            inputs=[xi, x], layer=Add, name='%s-Add' % attention_name
+        )
+        x = self.apply(
+            inputs=x,
+            layer=LayerNormalization,
+            zero_mean=False,
+            scale=False,
+            offset=False,
+            name='%s-Norm' % attention_name
+        )
+
+        # Feed Forward
+        xi = x
+        x = self.apply(
+            inputs=x,
+            layer=FeedForward,
+            units=self.intermediate_size,
+            activation=self.hidden_act,
+            use_bias=False,
+            kernel_initializer=self.initializer,
+            name=feed_forward_name
+        )
+        x = self.apply(
+            inputs=x,
+            layer=Dropout,
+            rate=self.dropout_rate,
+            name='%s-Dropout' % feed_forward_name
+        )
+        x = self.apply(
+            inputs=[xi, x], layer=Add, name='%s-Add' % feed_forward_name
+        )
+        x = self.apply(
+            inputs=x,
+            layer=LayerNormalization,
+            zero_mean=False,
+            scale=False,
+            offset=False,
+            name='%s-Norm' % feed_forward_name
+        )
+
+        return x
+
+    def apply_final_layers(self, inputs):
+        """剩余部分
+        """
+        x = inputs
+
+        if self.with_mlm:
+            # 预测token概率部分
+            if self.embedding_size != self.hidden_size:
+                x = self.apply(
+                    inputs=x,
+                    layer=Dense,
+                    units=self.embedding_size,
+                    use_bias=False,
+                    kernel_initializer=self.initializer,
+                    name='Output-Mapping'
+                )
+            x = self.apply(
+                inputs=x,
+                layer=Dropout,
+                rate=self.dropout_rate,
+                name='Output-MLM-Dropout'
+            )
+            mlm_activation = 'softmax' if self.with_mlm is True else self.with_mlm
+            x = self.apply(
+                inputs=x,
+                layer=Embedding,
+                arguments={'mode': 'dense'},
+                name='Embedding-Token'
+            )
+            x = self.apply(
+                inputs=x,
+                layer=Activation,
+                activation=mlm_activation,
+                name='Output-MLM-Activation'
+            )
+
+        return x
+
+    def variable_mapping(self):
+        """删掉部分权重映射
+        """
+        mapping = super(RoFormerV2, self).variable_mapping()
+
+        for k, v in mapping.items():
+            v = [
+                i for i in v
+                if not string_matching(i, ['beta', 'gamma', 'bias'])
+            ]
+            mapping[k] = v
+
+        return mapping
+
+
 class ELECTRA(BERT):
     """Google推出的ELECTRA模型
     链接：https://arxiv.org/abs/2003.10555
@@ -1937,7 +2151,8 @@ class T5_Encoder(T5_Base):
         x = self.apply(
             inputs=self.simplify([x, z]),
             layer=LayerNormalization,
-            center=False,
+            zero_mean=False,
+            offset=False,
             epsilon=1e-6,
             conditional=(z is not None),
             hidden_units=self.layer_norm_conds[1],
@@ -1974,7 +2189,8 @@ class T5_Encoder(T5_Base):
         x = self.apply(
             inputs=self.simplify([x, z]),
             layer=LayerNormalization,
-            center=False,
+            zero_mean=False,
+            offset=False,
             epsilon=1e-6,
             conditional=(z is not None),
             hidden_units=self.layer_norm_conds[1],
@@ -2012,7 +2228,8 @@ class T5_Encoder(T5_Base):
         x = self.apply(
             inputs=self.simplify([x, z]),
             layer=LayerNormalization,
-            center=False,
+            zero_mean=False,
+            offset=False,
             epsilon=1e-6,
             conditional=(z is not None),
             hidden_units=self.layer_norm_conds[1],
@@ -2104,7 +2321,7 @@ class T5_Decoder(LM_Mask, T5_Base):
         return [c, x]
 
     def apply_main_layers(self, inputs, index):
-        """T5的Dencoder主体是基于Self-Attention、Cross-Attention的模块
+        """T5的Decoder主体是基于Self-Attention、Cross-Attention的模块
         顺序：LN --> Att1 --> Add --> LN --> Att2 --> Add -->  LN --> FFN --> Add
         """
         c, x = inputs
@@ -2121,7 +2338,8 @@ class T5_Decoder(LM_Mask, T5_Base):
         x = self.apply(
             inputs=self.simplify([x, z]),
             layer=LayerNormalization,
-            center=False,
+            zero_mean=False,
+            offset=False,
             epsilon=1e-6,
             conditional=(z is not None),
             hidden_units=self.layer_norm_conds[1],
@@ -2161,7 +2379,8 @@ class T5_Decoder(LM_Mask, T5_Base):
         x = self.apply(
             inputs=self.simplify([x, z]),
             layer=LayerNormalization,
-            center=False,
+            zero_mean=False,
+            offset=False,
             epsilon=1e-6,
             conditional=(z is not None),
             hidden_units=self.layer_norm_conds[1],
@@ -2201,7 +2420,8 @@ class T5_Decoder(LM_Mask, T5_Base):
         x = self.apply(
             inputs=self.simplify([x, z]),
             layer=LayerNormalization,
-            center=False,
+            zero_mean=False,
+            offset=False,
             epsilon=1e-6,
             conditional=(z is not None),
             hidden_units=self.layer_norm_conds[1],
@@ -2239,7 +2459,8 @@ class T5_Decoder(LM_Mask, T5_Base):
         x = self.apply(
             inputs=self.simplify([x, z]),
             layer=LayerNormalization,
-            center=False,
+            zero_mean=False,
+            offset=False,
             epsilon=1e-6,
             conditional=(z is not None),
             hidden_units=self.layer_norm_conds[1],
@@ -2255,8 +2476,9 @@ class T5_Decoder(LM_Mask, T5_Base):
         )
         x = self.apply(
             inputs=x,
-            layer=Scale,
+            layer=ScaleOffset,
             scale=self.hidden_size**(-0.5),
+            offset=False,
             name='Decoder-Output-Scale'
         )
 
@@ -2282,7 +2504,7 @@ class T5_Decoder(LM_Mask, T5_Base):
                     inputs=x,
                     layer=Activation,
                     activation=lm_activation,
-                    name='Dencoder-Output-LM-Activation'
+                    name='Decoder-Output-LM-Activation'
                 )
             else:
                 x = self.apply(
@@ -2453,6 +2675,7 @@ def build_transformer_model(
         'roberta': BERT,
         'nezha': NEZHA,
         'roformer': RoFormer,
+        'roformer_v2': RoFormerV2,
         'electra': ELECTRA,
         'gpt': GPT,
         'gpt2': GPT2,
@@ -2474,8 +2697,10 @@ def build_transformer_model(
     if is_string(model):
         model = model.lower()
         MODEL = models[model]
-        if model.endswith('t5.1.1'):
-            configs['version'] = model
+        if model.startswith('t5.1.1'):
+            configs['version'] = 't5.1.1'
+        elif model.startswith('mt5.1.1'):
+            configs['version'] = 'mt5.1.1'
     else:
         MODEL = model
 
